@@ -47,6 +47,7 @@
 
 /* Internal includes, do not use in your programs */
 #include <OpenIPMI/internal/ipmi_malloc.h>
+#include "cmdlang.h"
 
 static void
 mc_list_handler(ipmi_domain_t *domain, ipmi_mc_t *mc, void *cb_data)
@@ -1592,7 +1593,7 @@ got_users(ipmi_mc_t        *mc,
     ipmi_cmdlang_t  *cmdlang = ipmi_cmdinfo_get_cmdlang(cmd_info);
     ipmi_user_t     *user;
     char            mc_name[IPMI_MC_NAME_LEN];
-    unsigned int    i, j, k;
+    int		    i, j, k;
     char            str[17];
     unsigned int    count;
     unsigned int    channel;
@@ -1628,7 +1629,7 @@ got_users(ipmi_mc_t        *mc,
     rv = ipmi_user_list_get_fixed_users(list, &val);
     if (!rv)
 	ipmi_cmdlang_out_int(cmd_info, "Fixed Users", val);
-    for (i=0; i<count; i++) {
+    for (i=0; i<(int)count; i++) {
 	user = ipmi_user_list_get_user(list, i);
 	if (!user)
 	    continue;
@@ -1997,7 +1998,8 @@ mc_user_set(ipmi_mc_t *mc, void *cb_data)
 		cmdlang->errstr = "no name value";
 		goto out_err;
 	    }
-	    strncpy(info->name, argv[curr_arg], 16);
+	    strncpy(info->name, argv[curr_arg], 15);
+	    info->name[15] = '\0';
 	} else if (strcmp(argv[curr_arg], "password") == 0) {
 	    info->pw_set = 1;
 	    curr_arg++;
@@ -2015,7 +2017,8 @@ mc_user_set(ipmi_mc_t *mc, void *cb_data)
 		cmdlang->errstr = "no name value";
 		goto out_err;
 	    }
-	    strncpy(info->pw, argv[curr_arg], 20);
+	    strncpy(info->pw, argv[curr_arg], 19);
+	    info->pw[19] = '\0';
 	} else {
 	    cmdlang->err = EINVAL;
 	    cmdlang->errstr = "invalid setting";
